// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012 Mario Guerriero <mefrio.g@gmail.com>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;

using Smiler.Utils;

namespace Smiler {

    public class EffectManager : GLib.Object {
        
        public signal void effect_loaded (Effect effect);
        
        public GLib.List<Effect> effects = new GLib.List<Effect> ();
    
    	public EffectManager () {
    	    // Create all the effects
    	    var eff = new Effect (_("Bulge"), "bulge", EFFECTDIR + "bulge.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Cartoon"), "frei0r-filter-cartoon", EFFECTDIR + "cartoon.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Che Guevara"), "frei0r-filter-twolay0r", EFFECTDIR + "cheguevara.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Chrome"), "frei0r-filter-color-distance", EFFECTDIR + "chrome.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Dice TV"), "dicetv", EFFECTDIR + "dicetv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Distortion"), "frei0r-filter-distort0r", EFFECTDIR + "distortion.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Edge TV"), "edgetv", EFFECTDIR + "edgetv.jpg");
    	    effects.append (eff);
    	    /*eff = new Effect (_("Flip"), "videoflip method=horizontal-flip", EFFECTDIR + "flip.jpg");
    	    effects.append (eff);*/
    	    eff = new Effect (_("Heat"), "coloreffects preset=heat", EFFECTDIR + "heat.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Historical"), "videobalance saturation=0 ! agingtv", EFFECTDIR + "historical.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Hulk"), "videobalance saturation=1.5 hue=-0.5", EFFECTDIR + "hulk.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Invertion"), "frei0r-filter-invert0r", EFFECTDIR + "invertion.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Kaleidoscope"), "kaleidoscope", EFFECTDIR + "kaleidoscope.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Mauve"), "videobalance saturation=1.5 hue=+0.5", EFFECTDIR + "mauve.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Mirror"), "mirror", EFFECTDIR + "mirror.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Noir"), "videobalance saturation=0", EFFECTDIR + "noir.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Optical Illusion"), "optv", EFFECTDIR + "optv.jpg");
    	    effects.append (eff);
    	    /*eff = new Effect (_("Pinch"), "pinch", EFFECTDIR + "pinch.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Quark"), "quarktv", EFFECTDIR + "quarktv.jpg");
    	    effects.append (eff);*/
    	    eff = new Effect (_("Radioactive"), "radioactv", EFFECTDIR + "radioactv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Waveform"), "revtv", EFFECTDIR + "revtv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Ripple"), "rippletv", EFFECTDIR + "rippletv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Saturation"), "videobalance saturation=2", EFFECTDIR + "saturation.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Sepia"), "coloreffects preset=sepia", EFFECTDIR + "sepia.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Shagadelic"), "shagadelictv", EFFECTDIR + "shagadelictv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Sobel"), "frei0r-filter-sobel", EFFECTDIR + "sobel.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Square"), "square", EFFECTDIR + "square.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Kung-Fu"), "streaktv", EFFECTDIR + "streaktv.jpg");
    	    effects.append (eff);
    	    eff = new Effect (_("Stretch"), "stretch", EFFECTDIR + "stretch.jpg");
    	    effects.append (eff);
    	    /*eff = new Effect (_("Time delay"), "frei0r-filter-delay0r delaytime=5", EFFECTDIR + "timedelay.jpg");
    	    effects.append (eff);*/
    	    eff = new Effect (_("Twirl"), "twirl", EFFECTDIR + "twirl.jpg");
    	    effects.append (eff);
    	    /*eff = new Effect (_("Vertigo"), "vertigotv", EFFECTDIR + "vertigotv.jpg");
    	    effects.append (eff);*/
    	    eff = new Effect (_("X-Ray"), "coloreffects preset=xray", EFFECTDIR + "xray.jpg");
    	    effects.append (eff);
    	}
    	
    	public void apply_effect (Effect effect, string origin, string dest) {
            try {
                string dec = null;
        	    string enc = null;
        	    if (".jpg" in dest || ".jpeg" in dest) {
        	        dec = "jpegdec";
        	        enc = "jpegenc";
        	    }
        	    else if (".png" in dest) {
        	        dec = "pngdec";
        	        enc = "pngenc";
        	    }
        	    string exec = "gst-launch filesrc location=\"" + origin + "\" ! " + dec + " ! ffmpegcolorspace ! " + effect.pipename + " ! ffmpegcolorspace ! " + enc + " ! filesink location=\"" + dest + "\"";
    	        GLib.Process.spawn_command_line_sync (exec);   
    	    } catch (SpawnError e) {
    	        stderr.printf ("%s\n", e.message);
            }
    	    effect_loaded (effect);
    	}
    }
}
